/*
 * skillchains-db-xul.js
 *
 * Copyright (c) 2004 by Daniel Potter.  All rights reserved.
 *
 * Usage of this file falls under the terms of the GPL.
 *
 * Controls the XUL database system.
 */

function initDatabase() {
    try {
	// Start building the tree view of the database...
	var tree = document.getElementById("weaponsTreeChildren");
	var container, item, row, cell, elements;
	for (var w in SkillChainDB.weapons) {
	    item = document.createElement("treeitem");
	    item.setAttribute("container", "true");
	    item.setAttribute("open", "false");
	    row = document.createElement("treerow");
	    item.appendChild(row);
	    cell = document.createElement("treecell");
	    cell.setAttribute("label", SkillChainDB.weapons[w].name);
	    row.appendChild(cell);
	    container = document.createElement("treechildren");
	    item.appendChild(container);
	    tree.appendChild(item);
	    // now add the jobs
	    appendJobs(container, getLocalString("exjobs"), SkillChainDB.weapons[w].ex);
	    appendJobs(container, getLocalString("spjobs"), SkillChainDB.weapons[w].sp);
	    // now the skills (what everyone really wants to see :) )
	    item = document.createElement("treeitem");
	    container.appendChild(item);
	    item.setAttribute("container", "true");
	    item.setAttribute("open", "true");
	    row = document.createElement("treerow");
	    item.appendChild(row);
	    cell = document.createElement("treecell");
	    cell.setAttribute("label", getLocalString("weapon.skills"));
	    row.appendChild(cell);
	    container = document.createElement("treechildren");
	    item.appendChild(container);
	    for (var s in SkillChainDB.weapons[w].skills) {
		item = document.createElement("treeitem");
		container.appendChild(item);
		item.setAttribute("container", "false");
		row = document.createElement("treerow");
		item.appendChild(row);
		cell = document.createElement("treecell");
		cell.setAttribute("label", SkillChainDB.weapons[w].skills[s].name);
		row.appendChild(cell);
		// first element
		cell = document.createElement("treecell");
		if (SkillChainDB.weapons[w].skills[s].element == null) {
		    // no element (can't be used in a chain)
		    cell.setAttribute("label", "-");
		    elements = getLocalString("none");
		} else {
		    cell.setAttribute("src", icon(SkillChainDB.weapons[w].skills[s].element.icon));
		    elements = SkillChainDB.weapons[w].skills[s].element.name;
		}
		row.appendChild(cell);
		// second element
		cell = document.createElement("treecell");
		if (SkillChainDB.weapons[w].skills[s].element2 != null) {
		    cell.setAttribute("src", icon(SkillChainDB.weapons[w].skills[s].element2.icon));
		    elements += ", " + SkillChainDB.weapons[w].skills[s].element2.name;
		}
		row.appendChild(cell);
		// third element
		cell = document.createElement("treecell");
		if (SkillChainDB.weapons[w].skills[s].element3 != null) {
		    cell.setAttribute("src", icon(SkillChainDB.weapons[w].skills[s].element3.icon));
		    elements += ", " + SkillChainDB.weapons[w].skills[s].element3.name;
		}
		row.appendChild(cell);
		cell = document.createElement("treecell");
		cell.setAttribute("label", SkillChainDB.weapons[w].skills[s].level);
		row.appendChild(cell);
		cell = document.createElement("treecell");
		var flags = SkillChainDB.weapons[w].skills[s].flags;
		if (flags == WS_SP) {
		    cell.setAttribute("label", getLocalString("sp"));
		} else if (flags == WS_EX) {
		    cell.setAttribute("label", getLocalString("ex"));
		} else {
		    cell.setAttribute("label", "");
		}
		row.appendChild(cell);
		cell = document.createElement("treecell");
		cell.setAttribute("label", elements);
		row.appendChild(cell);
	    }
	}
	// Now: populate the elements
	tree = document.getElementById("elementsTreeChildren");
	var any = getLocalString("any");
	for (var e in SkillChainDB.elements) {
	    item = document.createElement("treeitem");
	    item.setAttribute("container", "true");
	    item.setAttribute("open", "false");
	    row = document.createElement("treerow");
	    item.appendChild(row);
	    cell = document.createElement("treecell");
	    cell.setAttribute("label", SkillChainDB.elements[e].name);
	    cell.setAttribute("src", icon(SkillChainDB.elements[e].icon));
	    row.appendChild(cell);
	    container = document.createElement("treechildren");
	    item.appendChild(container);
	    tree.appendChild(item);
	    // now add the elements this chains with
	    for (c in SkillChainDB.elements[e].chains) {
		item = document.createElement("treeitem");
		container.appendChild(item);
		item.setAttribute("container", "false");
		row = document.createElement("treerow");
		item.appendChild(row);
		cell = document.createElement("treecell");
		cell.setAttribute("label", SkillChainDB.elements[e].name);
		cell.setAttribute("src", icon(SkillChainDB.elements[e].icon));
		row.appendChild(cell);
		cell = document.createElement("treecell");
		cell.setAttribute("label", SkillChainDB.elements[e].chains[c].toElement.name);
		cell.setAttribute("src", icon(SkillChainDB.elements[e].chains[c].toElement.icon));
		row.appendChild(cell);
		cell = document.createElement("treecell");
		cell.setAttribute("label", SkillChainDB.elements[e].chains[c].chain.name);
		cell.setAttribute("src", icon(SkillChainDB.elements[e].chains[c].chain.icon));
		row.appendChild(cell);
		cell = document.createElement("treecell");
		if (SkillChainDB.elements[e].chains[c].prevChain == null) {
		    cell.setAttribute("label", any);
		} else {
		    cell.setAttribute("label", SkillChainDB.elements[e].chains[c].prevChain.name);
		    cell.setAttribute("src", icon(SkillChainDB.elements[e].chains[c].prevChain.icon));
		}
		row.appendChild(cell);
	    }
	}
    } catch (ex) {
	alert("d'oh: " + ex);
    }
}

function icon(name) {
    return "chrome://skillchains/skin/" + name + ".png";
}

function appendJobs(container, label, jobs) {
    var item = document.createElement("treeitem");
    container.appendChild(item);
    if (jobs != null && jobs.length >= 1) {
	item.setAttribute("container", "true");
	item.setAttribute("open", "true");
	var row = document.createElement("treerow");
	item.appendChild(row);
	var cell = document.createElement("treecell");
	cell.setAttribute("label", label);
	row.appendChild(cell);
	var children = document.createElement("treechildren");
	item.appendChild(children);
	for (var j in jobs) {
	    item = document.createElement("treeitem");
	    row = document.createElement("treerow");
	    item.appendChild(row);
	    cell = document.createElement("treecell");
	    cell.setAttribute("label", jobs[j].name);
	    row.appendChild(cell);
	    children.appendChild(item);
	}
    } else {
	var row = document.createElement("treerow");
	item.appendChild(row);
	var cell = document.createElement("treecell");
	cell.setAttribute("label", label + " (" + getLocalString("none") + ")");
	row.appendChild(cell);
    }
}
