/*
 * skillchains.js
 *
 * Copyright (c) 2004 by Daniel Potter.  All rights reserved.
 *
 * Usage of this file falls under the terms of the GPL.
 *
 * Basically, this defines the data model for the skillchains as well as an
 * API for using the data model.  It also creates the skillchains database.
 */

// Current version - this is simply incremented for every update
var SKILLCHAIN_VERSION = 7;

function debug(msg) {
//    document.forms.skillchains.console.value += "\r\n" + msg;
}

// Structure for a Skill Chain.  (Ultimate Renkeis have four elements.)
function Chain(name, icon, level, chainElement1, chainElement2, chainElement3, chainElement4) {
    if (name == null || level == null || chainElement1 == null)
	throw "Missing required values for new chain";
    this.name = name;
    this.icon = icon;
    this.level = level;
    this.chainElement1 = chainElement1;
    this.chainElement2 = chainElement2;
    this.chainElement3 = chainElement3;
    this.chainElement4 = chainElement4;
}

function ChainLink(chain, fromElement, toElement, prevChain) {
    if (chain == null || fromElement == null || toElement == null)
	throw "Missing values linking chains.";
    this.chain = chain;
    this.fromElement = fromElement;
    this.toElement = toElement;
    this.prevChain = prevChain;
}

function Element(name, icon) {
    this.name = name;
    this.icon = icon;
    this.chains = new Array();
    this.addChain = function(chain, element, prevChain) {
	this.chains.push(new ChainLink(chain, this, element, prevChain));
    }
    // Checks to see which chain, if any, can be made going from
    // this element to the other element.
    // otherElement - the other element
    // prevChain    - the previous chain, or null if this is the start of a chain
    this.findChain = function(otherElement, prevChain) {
	with (this) {
	    var i;
	    for (i = 0; i < chains.length; i++) {
		if (chains[i].toElement == otherElement) {
		    if (chains[i].prevChain != null) {
			if (chains[i].prevChain != prevChain) {
			    continue;
			}
		    }
		    return chains[i].chain;
		}
	    }
	}
	return null;
    }
}

var WS_NORMAL = 0;
var WS_EX = 1;
var WS_SP = 2;

function WeaponSkill(name, level, flags, element, element2, element3) {
    if (name == null || level == null)
	throw "Missing required values for Weapon Skill";
    if (flags != 0 && flags != 1 && flags != 2)
	throw "Invalid flags " + flags + " for skill";
    this.name = name;
    this.flags = flags;
    this.level = level;
    this.element = element;
    this.element2 = element2;
    this.element3 = element3;
    // Find a chain (if any) *FROM* this skill *TO* the given skill
    this.findChainTo = function(toSkill, prevChain) {
	// Find a chain with the other weapon.  Our FIRST
	// element always take priority, then down the chain
	with (this) {
	    // some skills have no elements
	    // they're listed so they appear in the DB, but they can't be
	    // used to chain
	    if (element == null)
		return null;
	    var chain = element.findChain(toSkill.element, prevChain);
	    if (chain == null && toSkill.element2 != null) {
		chain = element.findChain(toSkill.element2, prevChain);
	    }
	    if (chain == null && toSkill.element3 != null) {
		chain = element.findChain(toSkill.element3, prevChain);
	    }
	    if (chain == null && element2 != null) {
		chain = element2.findChain(toSkill.element, prevChain);
		if (chain == null && toSkill.element2 != null) {
		    chain = element2.findChain(toSkill.element2, prevChain);
		}
		if (chain == null && toSkill.element3 != null) {
		    chain = element2.findChain(toSkill.element3, prevChain);
		}
	    }
	    if (chain == null && element3 != null) {
		chain = element3.findChain(toSkill.element, prevChain);
		if (chain == null && toSkill.element2 != null) {
		    chain = element3.findChain(toSkill.element2, prevChain);
		}
		if (chain == null && toSkill.element3 != null) {
		    chain = element3.findChain(toSkill.element3, prevChain);
		}
	    }
	    return chain;
	}
    }
}

function SkillChain(fromSkill, toSkill, chain) {
    this.name = chain.name;
    this.fromSkill = fromSkill;
    this.toSkill = toSkill;
    this.chain = chain;
    // the number of chains in this chain (inclusive - ie, by default, 1)
    this.chainNumber = 1;
    // the next chain to traverse to (or null if there is no next chain)
    this.nextChain = null;
    // the previous chain (used when nesting to make sure the counts
    this.prevChain = null;
    // Clone this skillchain (including all nested chains)
    this.clone = function() {
	var chain = new SkillChain(this.fromSkill, this.toSkill, this.chain);
	chain.chainNumber = this.chainNumber;
	if (this.nextChain != null) {
	    chain.nextChain = this.nextChain.clone();
	    chain.nextChain.prevChain = chain;
	}
	if (this.prevChain != null) {
	    // cloning the previous chains is kinda weird...
	    // we go up, creating a duplicate copy of every chain until
	    // we find the parent.  Then we correct the chain count up
	    // the list.
	    var curNextChain = chain; // the cloned chain that follows the current chain
	    var curChain = this.prevChain; // the chain being cloned
	    var nChain; // the cloned chain
	    while (curChain != null) {
		// we can't directly clone that chain, instead do a simple clone
		nChain = new SkillChain(curChain.fromSkill, curChain.toSkill, curChain.chain);
		// ok, now set up the linked list...
		nChain.nextChain = curNextChain;
		curNextChain.prevChain = nChain;
		// move up the list
		curChain = curChain.prevChain;
		curNextChain = nChain;
	    }
	    // now correct the counts
	    chain.correctCount(chain.chainNumber);
	}
	return chain;
    }
    this.nest = function(chain) {
	this.nextChain = chain;
	chain.prevChain = this;
	this.chainNumber = chain.chainNumber + 1;
	if (this.prevChain != null)
	    this.prevChain.correctCount(this.chainNumber + 1);
	this.name = this.chain.name + " " + getLocalString("arrow") + " " + chain.name;
    }
    this.getStarter = function() {
	var starter = this;
	while (starter.prevChain != null) {
	    starter = starter.prevChain;
	}
	return starter;
    }
    this.correctCount = function(count) {
	this.chainNumber = count;
	// while we're correcting the count, correct our name, too
	this.name = this.chain.name + " " + getLocalString("arrow") + " " + this.nextChain.name;
	if (this.prevChain != null)
	    this.prevChain.correctCount(count + 1);
    }
    // completely compares this chain to the other chain (ignoring any
    // previous chain info)
    this.equals = function(otherChain) {
	if (this.fromSkill != otherChain.fromSkill ||
	    this.toSkill != otherChain.toSkill ||
	    this.chain != otherChain.chain) {
	    return false;
	}
	if (this.nextChain == null)
	    return true;
	// recurse down (tail recursion would be cool)
	return this.nextChain.equals(otherChain.nextChain);
    }
}

// Simple job object - not much more than a string, but...
function Job(name, abbr) {
    this.name = name;
    this.abbr = abbr;
}

function Weapon(name, ex, sp) {
    this.name = name;
    this.ex = ex;
    this.sp = sp;
    this.skills = new Array();
    this.addSkill = function(skill) {
	skill.weapon = this;
	this.skills.push(skill);
    }
    // Find a chain with another weapon
    // otherWeapon - the weapon to find a chain TO
    // maxLevel    - the maximum skill level available on either weapon
    // prevChain   - the previous chain, or null if this is the start
    this.findChains = function(otherWeapon, maxLevel, prevChain) {
	if (maxLevel == null)
	    maxLevel = 500;
	with (this) {
	    var i, j;
	    var chains = new Array();
	    for (i = 0; i < skills.length; i++) {
		if (skills[i].level <= maxLevel && skills[i].element != null) {
		    for (j = 0; j < otherWeapon.skills.length; j++) {
			if (otherWeapon.skills[j].level <= maxLevel && otherWeapon.skills[j].element != null) {
			    var chain = skills[i].findChainTo(otherWeapon.skills[j], prevChain);
			    if (chain != null) {
				chains.push(new SkillChain(skills[i], otherWeapon.skills[j], chain));
			    }
			}
		    }
		}
	    }
	    return chains;
	}
    }
}

// The player object represents a player with a given weapon at a given
// skill level
//   weapon     - the weapon being used
//   skillLevel - the maximum skill level we can use
//   ex         - true if we can use EX skills
//   sp         - true if we can use SP skills
function Player(weapon, skillLevel, ex, sp) {
    this.weapon = weapon;
    this.skillLevel = skillLevel;
    this.ex = ex;
    this.sp = sp;
    // we start out active
    this.active = true;
    // Can we use this skill? (NOTE: does *not* check to see if the skill
    // comes from the weapon used.)
    this.canUse = function(skill) {
	if (skill.level <= this.skillLevel) {
	    if (skill.flags == WS_EX) {
		return this.ex;
	    } else if (skill.flags == WS_SP) {
		return this.sp;
	    }
	    return true;
	}
	return false;
    }
    // Finds all chains possible *FROM* us *TO* the other player
    // otherPlayer - the other player
    // prevChain   - the previous chain
    this.findChains = function(otherPlayer, prevChain) {
	with (this) {
	    var chains = new Array();
	    for (var i in weapon.skills) {
		// can we use this skill?
		if (canUse(weapon.skills[i])) {
		    for (var j in otherPlayer.weapon.skills) {
			if (otherPlayer.canUse(otherPlayer.weapon.skills[j])) {
			    var chain = weapon.skills[i].findChainTo(otherPlayer.weapon.skills[j], prevChain);
			    if (chain != null) {
				chains.push(new SkillChain(weapon.skills[i], otherPlayer.weapon.skills[j], chain));
			    }
			}
		    }
		}
	    }
	    return chains;
	}
    }
    // Find chains FROM a weapon skill TO this player
    this.findChainsFromWeaponSkill = function(weaponSkill, prevChain) {
	with (this) {
	    var chains = new Array();
	    for (var i in weapon.skills) {
		// can we use this skill?
		if (canUse(weapon.skills[i])) {
		    var chain = weaponSkill.findChainTo(weapon.skills[i], prevChain);
		    if (chain != null) {
			chains.push(new SkillChain(weaponSkill, weapon.skills[i], chain));
		    }
		}
	    }
	    return chains;
	}
    }
}

// Compares two skillchains (most useful for sorting - Array.sort(skillChainCompare)).
// Skill chains are first compared by the chain number, then chain type
// (chains with one element sort earlier than chains with two, etc.), then by
// the chain type's name, then by the first skill's name, then the second skill's name.
function skillChainCompare(a, b) {
    // Sort first by chain number...
    if (a.chainNumber != b.chainNumber) {
	return a.chainNumber - b.chainNumber;
    }
    // Then by chain...
    if (a.chain != b.chain) {
	// Chains with fewer elements go first
	if (a.chain.chainElement4 == null && b.chain.chainElement4 != null)
	    return -1;
	if (a.chain.chainElement4 != null && b.chain.chainElement4 == null)
	    return 1;
	if (a.chain.chainElement2 == null && b.chain.chainElement2 != null)
	    return -1;
	if (a.chain.chainElement2 != null && b.chain.chainElement2 == null)
	    return 1;
	return a.chain.name < b.chain.name ? -1 : (a.chain.name == b.chain.name ? 0 : 1);
    } else {
	// then by weapon skill
	var aSkill = a.fromSkill, bSkill = b.fromSkill;
	if (aSkill == bSkill) {
	    aSkill = a.toSkill;
	    bSkill = b.toSkill;
	}
	return aSkill.name < bSkill.name ? -1 : (aSkill.name == bSkill.name ? 0 : 1);
    }
}

/*
 * Compares two object by their name field
 */
function compareNames(a, b) {
    return a.name < b.name ? -1 : (a.name == b.name ? 0 : 1);
}

/*
 *
 * START OF DATABASE
 *
 * All code below here creates the data for the database.
 *
 */

// Define the various elements

var EARTH = new Element(getLocalString("earth"), "earth");
var THUNDER = new Element(getLocalString("thunder"), "thunder");
var WATER = new Element(getLocalString("water"), "water");
var FIRE = new Element(getLocalString("fire"), "fire");
var ICE = new Element(getLocalString("ice"), "ice");
var WIND = new Element(getLocalString("wind"), "wind");
var LIGHT = new Element(getLocalString("light"), "light");
var DARK = new Element(getLocalString("dark"), "dark");

// NOTE: These are not *really* elements.  They mark the various Ultimate
// Renkeis.  Certain weapon skills, instead of having an element, form
// the ultimate renkei.  These mark those weapon skills.

var UE_FRAGMENTATION = new Element(getLocalString("ue.fragmentation"), "ufragmentation");
var UE_FUSION = new Element(getLocalString("ue.fusion"), "ufusion");
var UE_GRAVITATION = new Element(getLocalString("ue.gravitation"), "ugravitation");
var UE_DISTORTION = new Element(getLocalString("ue.distortion"), "udistortion");

// Create the chains, now

// Level 1 chains:
var SCISSION = new Chain(getLocalString("scission"), "scission", 1, EARTH);
var IMPACTION = new Chain(getLocalString("impaction"), "impaction", 1, THUNDER);
var REVERBERATION = new Chain(getLocalString("reverberation"), "reverberation", 1, WATER);
var LIQUEFACTION = new Chain(getLocalString("liquefaction"), "liquefaction", 1, FIRE);
var INDURATION = new Chain(getLocalString("induration"), "induration", 1, ICE);
var DETONATION = new Chain(getLocalString("detonation"), "detonation", 1, WIND);
var TRANSFIXION = new Chain(getLocalString("transfixion"), "transfixion", 1, LIGHT);
var COMPRESSION = new Chain(getLocalString("compression"), "compression", 1, DARK);

// Level 2 chains:
var GRAVITATION = new Chain(getLocalString("gravitation"), "gravitation", 2, DARK, EARTH);
var DISTORTION = new Chain(getLocalString("distortion"), "distortion", 2, WATER, ICE);
var FUSION = new Chain(getLocalString("fusion"), "fusion", 2, FIRE, LIGHT);
var FRAGMENTATION = new Chain(getLocalString("fragmentation"), "fragmentation", 2, THUNDER, WIND);

// Level 3 chains:

var ULTIMATE_LIGHT = new Chain(getLocalString("ultimate.light"), "ultlight", 3, FIRE, LIGHT, THUNDER, WIND);
var ULTIMATE_DARKNESS = new Chain(getLocalString("ultimate.darkness"), "ultdark", 3, EARTH, DARK, WATER, ICE);

// Link the elements now to describe how chains can be formed

EARTH.addChain(REVERBERATION, WATER);
EARTH.addChain(DETONATION, WIND);
EARTH.addChain(LIQUEFACTION, FIRE);
// NOTE: only when coming from Distortion
EARTH.addChain(FUSION, UE_FUSION, DISTORTION);
// NOTE: only when coming from Distortion
EARTH.addChain(ULTIMATE_DARKNESS, UE_GRAVITATION, DISTORTION);
THUNDER.addChain(LIQUEFACTION, FIRE);
THUNDER.addChain(DETONATION, WIND);
// NOTE: only when coming from Fusion
THUNDER.addChain(GRAVITATION, UE_GRAVITATION, FUSION);
// NOTE: only when coming from Fusion
THUNDER.addChain(ULTIMATE_LIGHT, UE_FRAGMENTATION, FUSION);
WATER.addChain(INDURATION, ICE);
WATER.addChain(IMPACTION, THUNDER);
// NOTE: only when coming from Fragmentation
WATER.addChain(DISTORTION, UE_DISTORTION, FRAGMENTATION);
// NOTE: only when coming from Fragmentation
WATER.addChain(ULTIMATE_LIGHT, UE_FUSION, FRAGMENTATION);
FIRE.addChain(SCISSION, EARTH);
FIRE.addChain(FUSION, THUNDER);
ICE.addChain(FRAGMENTATION, WATER);
ICE.addChain(IMPACTION, THUNDER);
ICE.addChain(COMPRESSION, DARK);
WIND.addChain(SCISSION, EARTH);
WIND.addChain(GRAVITATION, DARK);
LIGHT.addChain(REVERBERATION, WATER);
LIGHT.addChain(DISTORTION, EARTH);
LIGHT.addChain(COMPRESSION, DARK);
DARK.addChain(DETONATION, WIND);
DARK.addChain(TRANSFIXION, LIGHT);
// NOTE: only when coming from Gravitation
DARK.addChain(FRAGMENTATION, UE_FRAGMENTATION, GRAVITATION);
// NOTE: only when coming from Gravitation
DARK.addChain(ULTIMATE_DARKNESS, UE_DISTORTION, GRAVITATION);

UE_FUSION.addChain(GRAVITATION, UE_GRAVITATION);
UE_FUSION.addChain(ULTIMATE_LIGHT, UE_FRAGMENTATION);
UE_FRAGMENTATION.addChain(DISTORTION, UE_DISTORTION);
UE_FRAGMENTATION.addChain(ULTIMATE_LIGHT, UE_FUSION);
UE_DISTORTION.addChain(FUSION, UE_FUSION);
UE_DISTORTION.addChain(ULTIMATE_DARKNESS, UE_GRAVITATION);
UE_GRAVITATION.addChain(FRAGMENTATION, UE_FRAGMENTATION);
UE_GRAVITATION.addChain(ULTIMATE_DARKNESS, UE_DISTORTION);

// Create the jobs:

var WARRIOR = new Job(getLocalString("warrior"), getLocalString("warrior.abrv"));
var THIEF = new Job(getLocalString("thief"), getLocalString("thief.abrv"));
var MONK = new Job(getLocalString("monk"), getLocalString("monk.abrv"));
var RED_MAGE = new Job(getLocalString("red.mage"), getLocalString("red.mage.abrv"));
var WHITE_MAGE = new Job(getLocalString("white.mage"), getLocalString("white.mage.abrv"));
var BLACK_MAGE = new Job(getLocalString("black.mage"), getLocalString("black.mage.abrv"));
var BARD = new Job(getLocalString("bard"), getLocalString("bard.abrv"));
var BEASTMASTER = new Job(getLocalString("beastmaster"), getLocalString("beastmaster.abrv"));
var DARK_KNIGHT = new Job(getLocalString("dark.knight"), getLocalString("dark.knight.abrv"));
var DRAGOON = new Job(getLocalString("dragoon"), getLocalString("dragoon.abrv"));
var NINJA = new Job(getLocalString("ninja"), getLocalString("ninja.abrv"));
var PALADIN = new Job(getLocalString("paladin"), getLocalString("paladin.abrv"));
var RANGER = new Job(getLocalString("ranger"), getLocalString("ranger.abrv"));
var SAMURAI = new Job(getLocalString("samurai"), getLocalString("samurai.abrv"));
var SUMMONER = new Job(getLocalString("summoner"), getLocalString("summoner.abrv"));

// OK, next the weapon classes...

var ARCHERY = new Weapon(getLocalString("archery"), [RANGER], [RANGER]);
var BOW = ARCHERY; // alias
var AXE = new Weapon(getLocalString("axe"), [WARRIOR, BEASTMASTER, DARK_KNIGHT], [WARRIOR, BEASTMASTER]);
var CLUB = new Weapon(getLocalString("club"), [WARRIOR, PALADIN, WHITE_MAGE, DARK_KNIGHT, SAMURAI], [WHITE_MAGE]);
var DAGGER = new Weapon(getLocalString("dagger"), [THIEF, RED_MAGE, BARD, RANGER, NINJA], [THIEF]);
var GREAT_AXE = new Weapon(getLocalString("great.axe"), [WARRIOR, DARK_KNIGHT], [WARRIOR]);
var GREAT_KATANA = new Weapon(getLocalString("great.katana"), null, [SAMURAI]);
var GREAT_SWORD = new Weapon(getLocalString("great.sword"), [WARRIOR, PALADIN, DARK_KNIGHT], [PALADIN, DARK_KNIGHT]);
var HAND_TO_HAND = new Weapon(getLocalString("hand.to.hand"), [MONK], [MONK]);
var KATANA = new Weapon(getLocalString("katana"), null, [NINJA]);
var MARKSMANSHIP = new Weapon(getLocalString("marksmanship"), [RANGER], [RANGER]);
var GUN = MARKSMANSHIP; // alias
var SCYTHE = new Weapon(getLocalString("scythe"), [WARRIOR, DARK_KNIGHT], [DARK_KNIGHT]);
var POLEARM = new Weapon(getLocalString("polearm"), [WARRIOR, PALADIN, DRAGOON], [DRAGOON]);
var SPEAR = POLEARM; // alias
var STAFF = new Weapon(getLocalString("staff"), [PALADIN, MONK, WARRIOR, WHITE_MAGE], null);
var SWORD = new Weapon(getLocalString("sword"), [PALADIN, WARRIOR, DARK_KNIGHT], [PALADIN]);

// Chuck the jobs and skills into arrays

var SkillChainDB = new Object();

SkillChainDB.jobs = [ WARRIOR, THIEF, MONK, RED_MAGE, WHITE_MAGE, BLACK_MAGE,
	BARD, BEASTMASTER, DARK_KNIGHT, DRAGOON, NINJA, PALADIN, RANGER,
	SAMURAI, SUMMONER ];
SkillChainDB.weapons = [ARCHERY, AXE, CLUB, DAGGER, GREAT_AXE, GREAT_KATANA,
	GREAT_SWORD, HAND_TO_HAND, KATANA, MARKSMANSHIP, SCYTHE, SPEAR,
	STAFF, SWORD];
SkillChainDB.elements = [ FIRE, ICE, WIND, EARTH, THUNDER, WATER, LIGHT, DARK, UE_FRAGMENTATION, UE_FUSION, UE_GRAVITATION, UE_DISTORTION ];

// be kind and sort these for the users
SkillChainDB.jobs.sort(compareNames);
SkillChainDB.weapons.sort(compareNames);

// Now, the skills themselves!
//
// NOTE: This skill list has NOT been verified.
// It is currently based on the list of weapon skills compiled in
// the Weapon Skill FAQ on Gamefaqs:
// <http://db.gamefaqs.com/computer/doswin/file/final_fantasy_xi_weapon_skillchain.txt>

// ARCHERY:
//  EX/SP - RNG
ARCHERY.addSkill(new WeaponSkill(getLocalString("flaming.arrow"), 10, WS_EX, FIRE, LIGHT)); // EX
ARCHERY.addSkill(new WeaponSkill(getLocalString("piercing.arrow"), 40, WS_EX, WATER, LIGHT)); // EX
ARCHERY.addSkill(new WeaponSkill(getLocalString("dulling.arrow"), 80, WS_EX, FIRE, LIGHT)); // EX
ARCHERY.addSkill(new WeaponSkill(getLocalString("sidewinder"), 175, WS_EX, WATER, LIGHT, WIND)); // EX
ARCHERY.addSkill(new WeaponSkill(getLocalString("blast.arrow"), 200, WS_SP, ICE, LIGHT)); // SP
ARCHERY.addSkill(new WeaponSkill(getLocalString("arching.arrow"), 225, WS_SP, UE_FUSION)); // SP
ARCHERY.addSkill(new WeaponSkill(getLocalString("emperial.arrow"), 250, WS_EX, UE_FUSION, LIGHT)); // EX
// AXE:
//  EX - WAR/BST/DRK
//  SP - WAR/BST
AXE.addSkill(new WeaponSkill(getLocalString("raging.axe"), 10, WS_NORMAL, WIND, THUNDER));
AXE.addSkill(new WeaponSkill(getLocalString("smash.axe"), 40, WS_NORMAL, ICE, WATER));
AXE.addSkill(new WeaponSkill(getLocalString("gale.axe"), 70, WS_NORMAL, WIND));
AXE.addSkill(new WeaponSkill(getLocalString("avalanche.axe"), 100, WS_NORMAL, EARTH, THUNDER));
AXE.addSkill(new WeaponSkill(getLocalString("spinning.axe"), 150, WS_EX, FIRE, EARTH, THUNDER)); // EX
AXE.addSkill(new WeaponSkill(getLocalString("rampage"), 175, WS_NORMAL, EARTH));
AXE.addSkill(new WeaponSkill(getLocalString("calamity"), 200, WS_SP, EARTH, THUNDER)); // SP
AXE.addSkill(new WeaponSkill(getLocalString("mistral.axe"), 225, WS_SP, UE_FUSION)); // SP
AXE.addSkill(new WeaponSkill(getLocalString("decimation"), 240, WS_NORMAL, UE_FUSION, WATER));
// CLUB:
//  EX - WAR/PLD/WHM/DRK/SAM
//  SP - WHM
CLUB.addSkill(new WeaponSkill(getLocalString("shining.strike"), 10, WS_NORMAL, THUNDER));
CLUB.addSkill(new WeaponSkill(getLocalString("seraph.strike"), 40, WS_EX, THUNDER)); // EX
CLUB.addSkill(new WeaponSkill(getLocalString("brainshaker"), 70, WS_NORMAL, WATER));
CLUB.addSkill(new WeaponSkill(getLocalString("star.light"), 100, WS_NORMAL));
CLUB.addSkill(new WeaponSkill(getLocalString("moon.light"), 100, WS_EX)); // EX
CLUB.addSkill(new WeaponSkill(getLocalString("skullbreaker"), 150, WS_NORMAL, ICE, WATER));
CLUB.addSkill(new WeaponSkill(getLocalString("true.strike"), 175, WS_NORMAL, WIND, THUNDER));
CLUB.addSkill(new WeaponSkill(getLocalString("judgment"), 200, WS_EX, THUNDER)); // EX
CLUB.addSkill(new WeaponSkill(getLocalString("hexa.strike"), 220, WS_SP, UE_FUSION)); // SP
CLUB.addSkill(new WeaponSkill(getLocalString("black.halo"), 230, WS_SP, UE_FRAGMENTATION, DARK)); // SP
// DAGGER:
//  EX - THF/RDM/BRD/RNG/NIN
//  SP - THF
DAGGER.addSkill(new WeaponSkill(getLocalString("wasp.sting"), 10, WS_NORMAL, EARTH));
DAGGER.addSkill(new WeaponSkill(getLocalString("gust.slash"), 40, WS_NORMAL, WIND));
DAGGER.addSkill(new WeaponSkill(getLocalString("shadowstich"), 70, WS_NORMAL, WATER));
DAGGER.addSkill(new WeaponSkill(getLocalString("viper.bite"), 100, WS_EX, EARTH)); // EX
DAGGER.addSkill(new WeaponSkill(getLocalString("cyclone"), 125, WS_EX, WIND, THUNDER)); // EX
DAGGER.addSkill(new WeaponSkill(getLocalString("energy.steal"), 150, WS_NORMAL));
DAGGER.addSkill(new WeaponSkill(getLocalString("energy.drain"), 175, WS_EX)); // EX
DAGGER.addSkill(new WeaponSkill(getLocalString("dancing.edge"), 200, WS_SP, EARTH, WIND)); // SP
DAGGER.addSkill(new WeaponSkill(getLocalString("shark.bite"), 225, WS_SP, UE_FRAGMENTATION)); // SP
DAGGER.addSkill(new WeaponSkill(getLocalString("evisceration"), 250, WS_NORMAL, UE_FRAGMENTATION, LIGHT));
// GREAT AXE:
//  EX - WAR/DRK
//  SP - WAR
GREAT_AXE.addSkill(new WeaponSkill(getLocalString("shield.break"), 10, WS_NORMAL, THUNDER));
GREAT_AXE.addSkill(new WeaponSkill(getLocalString("iron.tempest"), 40, WS_NORMAL, EARTH, THUNDER /* ? */));
GREAT_AXE.addSkill(new WeaponSkill(getLocalString("strumwind"), 70, WS_EX, WATER, EARTH)); // EX
GREAT_AXE.addSkill(new WeaponSkill(getLocalString("armor.break"), 100, WS_NORMAL, THUNDER));
GREAT_AXE.addSkill(new WeaponSkill(getLocalString("keen.edge"), 150, WS_NORMAL, DARK));
GREAT_AXE.addSkill(new WeaponSkill(getLocalString("weapon.break"), 175, WS_NORMAL, THUNDER));
GREAT_AXE.addSkill(new WeaponSkill(getLocalString("raging.rush"), 200, WS_SP, ICE, WATER)); // SP
GREAT_AXE.addSkill(new WeaponSkill(getLocalString("full.break"), 225, WS_SP, UE_DISTORTION)); // SP
GREAT_AXE.addSkill(new WeaponSkill(getLocalString("steel.cyclone"), 240, WS_NORMAL, UE_DISTORTION, WIND));
// GREAT KATANA:
//  SP - SAM
GREAT_KATANA.addSkill(new WeaponSkill(getLocalString("tachi.enpi"), 10, WS_NORMAL, LIGHT, EARTH));
GREAT_KATANA.addSkill(new WeaponSkill(getLocalString("tachi.hobaku"), 30, WS_NORMAL, ICE));
GREAT_KATANA.addSkill(new WeaponSkill(getLocalString("tachi.goten"), 70, WS_NORMAL, LIGHT, THUNDER));
GREAT_KATANA.addSkill(new WeaponSkill(getLocalString("tachi.kagero"), 100, WS_NORMAL, FIRE));
GREAT_KATANA.addSkill(new WeaponSkill(getLocalString("tachi.jinpu"), 150, WS_NORMAL, EARTH, WIND));
GREAT_KATANA.addSkill(new WeaponSkill(getLocalString("tachi.koki"), 175, WS_NORMAL, WATER, THUNDER)); // lements may be reversed on this
GREAT_KATANA.addSkill(new WeaponSkill(getLocalString("tachi.yukikaze"), 200, WS_SP, ICE, WIND)); // SP
GREAT_KATANA.addSkill(new WeaponSkill(getLocalString("tachi.gekko"), 225, WS_SP, UE_DISTORTION, WATER)); // SP
GREAT_KATANA.addSkill(new WeaponSkill(getLocalString("tachi.kasha"), 250, WS_NORMAL, UE_FUSION, DARK));
// GREAT SWORD
//  EX - WAR/PLD/DRK
//  SP - PLD/DRK
GREAT_SWORD.addSkill(new WeaponSkill(getLocalString("hard.slash"), 10, WS_NORMAL, EARTH));
GREAT_SWORD.addSkill(new WeaponSkill(getLocalString("power.slash"), 30, WS_NORMAL, LIGHT));
GREAT_SWORD.addSkill(new WeaponSkill(getLocalString("frostbite"), 70, WS_NORMAL, ICE));
GREAT_SWORD.addSkill(new WeaponSkill(getLocalString("freezebite"), 100, WS_EX, ICE, WIND)); // EX
GREAT_SWORD.addSkill(new WeaponSkill(getLocalString("shockwave"), 150, WS_NORMAL, WATER));
GREAT_SWORD.addSkill(new WeaponSkill(getLocalString("crescent.moon"), 175, WS_NORMAL, EARTH));
GREAT_SWORD.addSkill(new WeaponSkill(getLocalString("sickle.moon"), 200, WS_SP, EARTH, THUNDER)); // SP
GREAT_SWORD.addSkill(new WeaponSkill(getLocalString("spinning.slash"), 225, WS_SP, UE_GRAVITATION)); // SP
GREAT_SWORD.addSkill(new WeaponSkill(getLocalString("ground.strike"), 250, WS_NORMAL, UE_GRAVITATION, UE_DISTORTION));
// HAND TO HAND
//  EX - MNK
//  SP - MNK
HAND_TO_HAND.addSkill(new WeaponSkill(getLocalString("combo"), 10, WS_NORMAL, THUNDER));
HAND_TO_HAND.addSkill(new WeaponSkill(getLocalString("shoulder.tackle"), 40, WS_NORMAL, WATER, THUNDER));
HAND_TO_HAND.addSkill(new WeaponSkill(getLocalString("one.inch.punch"), 75, WS_EX, DARK)); // EX
HAND_TO_HAND.addSkill(new WeaponSkill(getLocalString("backhand.blow"), 100, WS_NORMAL, WIND));
HAND_TO_HAND.addSkill(new WeaponSkill(getLocalString("raging.fists"), 125, WS_EX, THUNDER)); // EX
HAND_TO_HAND.addSkill(new WeaponSkill(getLocalString("spinning.attack"), 150, WS_NORMAL, FIRE, THUNDER));
HAND_TO_HAND.addSkill(new WeaponSkill(getLocalString("howling.fist"), 200, WS_SP, LIGHT, THUNDER)); // SP
HAND_TO_HAND.addSkill(new WeaponSkill(getLocalString("dragon.kick"), 225, WS_SP, UE_FRAGMENTATION)); // SP
HAND_TO_HAND.addSkill(new WeaponSkill(getLocalString("ashuran.fist"), 250, WS_SP, UE_GRAVITATION, FIRE)); // SP
// KATANA
//  SP - NIN
KATANA.addSkill(new WeaponSkill(getLocalString("blade.rin"), 10, WS_NORMAL, LIGHT));
KATANA.addSkill(new WeaponSkill(getLocalString("blade.retsu"), 30, WS_NORMAL, EARTH));
KATANA.addSkill(new WeaponSkill(getLocalString("blade.teki"), 70, WS_NORMAL, WATER));
KATANA.addSkill(new WeaponSkill(getLocalString("blade.to"), 100, WS_NORMAL, ICE, WIND));
KATANA.addSkill(new WeaponSkill(getLocalString("blade.chi"), 150, WS_NORMAL, LIGHT, THUNDER));
KATANA.addSkill(new WeaponSkill(getLocalString("blade.ei"), 175, WS_NORMAL, DARK));
KATANA.addSkill(new WeaponSkill(getLocalString("blade.jin"), 200, WS_SP, WIND, THUNDER)); // SP
KATANA.addSkill(new WeaponSkill(getLocalString("blade.ten"), 225, WS_SP, UE_GRAVITATION)); // SP
KATANA.addSkill(new WeaponSkill(getLocalString("blade.ku"), 250, WS_SP, UE_GRAVITATION, LIGHT)); // SP
// MARKSMANSHIP
//  EX - RNG
//  SP - RNG
MARKSMANSHIP.addSkill(new WeaponSkill(getLocalString("hot.shot"), 10, WS_EX, FIRE, LIGHT)); // EX
MARKSMANSHIP.addSkill(new WeaponSkill(getLocalString("split.shot"), 40, WS_EX, WATER, LIGHT)); // EX
MARKSMANSHIP.addSkill(new WeaponSkill(getLocalString("sniper.shot"), 80, WS_EX, FIRE, LIGHT)); // EX
MARKSMANSHIP.addSkill(new WeaponSkill(getLocalString("slug.shot"), 175, WS_EX, WATER, LIGHT, WIND)); // EX
MARKSMANSHIP.addSkill(new WeaponSkill(getLocalString("blast.shot"), 200, WS_SP, ICE, LIGHT)); // SP
MARKSMANSHIP.addSkill(new WeaponSkill(getLocalString("heavy.shot"), 225, WS_SP, UE_FUSION)); // SP
MARKSMANSHIP.addSkill(new WeaponSkill(getLocalString("detonator"), 250, WS_NORMAL, UE_FUSION, LIGHT));
// SCYTHE
//  EX - WAR/DRK
//  SP - DRK
SCYTHE.addSkill(new WeaponSkill(getLocalString("slice"), 10, WS_NORMAL, EARTH));
SCYTHE.addSkill(new WeaponSkill(getLocalString("dark.harvest"), 40, WS_NORMAL, WATER));
SCYTHE.addSkill(new WeaponSkill(getLocalString("shadow.of.death"), 70, WS_EX, ICE, WATER)); // EX
SCYTHE.addSkill(new WeaponSkill(getLocalString("nightmare.scythe"), 100, WS_NORMAL, DARK, EARTH));
SCYTHE.addSkill(new WeaponSkill(getLocalString("spinning.scythe"), 125, WS_NORMAL, WATER, EARTH));
SCYTHE.addSkill(new WeaponSkill(getLocalString("vorpal.scythe"), 150, WS_NORMAL, LIGHT, EARTH));
SCYTHE.addSkill(new WeaponSkill(getLocalString("guillotine"), 200, WS_SP, ICE)); // SP
SCYTHE.addSkill(new WeaponSkill(getLocalString("cross.reaper"), 225, WS_SP, UE_DISTORTION)); // SP
SCYTHE.addSkill(new WeaponSkill(getLocalString("spiral.hell"), 240, WS_NORMAL, UE_DISTORTION, EARTH));
// SPEAR
//  EX - WAR/PLD/DRG
//  SP - DRG
SPEAR.addSkill(new WeaponSkill(getLocalString("double.thrust"), 10, WS_NORMAL, LIGHT));
SPEAR.addSkill(new WeaponSkill(getLocalString("thunder.thrust"), 30, WS_NORMAL, LIGHT, THUNDER));
SPEAR.addSkill(new WeaponSkill(getLocalString("raiden.thrust"), 70, WS_EX, LIGHT, THUNDER)); // EX
SPEAR.addSkill(new WeaponSkill(getLocalString("leg.sweep"), 100, WS_NORMAL, THUNDER));
SPEAR.addSkill(new WeaponSkill(getLocalString("penta.thrust"), 150, WS_NORMAL, DARK));
SPEAR.addSkill(new WeaponSkill(getLocalString("vorpal.thrust"), 175, WS_NORMAL, WATER, LIGHT));
SPEAR.addSkill(new WeaponSkill(getLocalString("skewer"), 200, WS_SP, DARK, LIGHT)); // SP
SPEAR.addSkill(new WeaponSkill(getLocalString("wheeling.thrust"), 225, WS_SP, UE_FUSION)); // SP
SPEAR.addSkill(new WeaponSkill(getLocalString("impulse.drive"), 240, WS_NORMAL, UE_GRAVITATION, ICE /* ? */));
// STAFF
//  EX - PLD/MNK/WAR/WHM
STAFF.addSkill(new WeaponSkill(getLocalString("heavy.swing"), 10, WS_NORMAL, THUNDER));
STAFF.addSkill(new WeaponSkill(getLocalString("rock.crusher"), 40, WS_NORMAL, THUNDER));
STAFF.addSkill(new WeaponSkill(getLocalString("earth.crusher"), 70, WS_EX, WIND, THUNDER)); // EX
STAFF.addSkill(new WeaponSkill(getLocalString("star.burst"), 100, WS_NORMAL, DARK, WATER));
STAFF.addSkill(new WeaponSkill(getLocalString("sun.burst"), 150, WS_EX, DARK, WATER)); // EX
STAFF.addSkill(new WeaponSkill(getLocalString("shell.crusher"), 175, WS_NORMAL, WIND, THUNDER));
STAFF.addSkill(new WeaponSkill(getLocalString("full.swing"), 200, WS_NORMAL, FIRE, THUNDER));
STAFF.addSkill(new WeaponSkill(getLocalString("spirit.taker"), 215, WS_NORMAL, THUNDER, FIRE)); // r possibly the other way arround
STAFF.addSkill(new WeaponSkill(getLocalString("retribution"), 230, WS_NORMAL, UE_GRAVITATION, WATER));
// SWORD
//  EX - PLD/WAR/DRK
//  SP - PLD
SWORD.addSkill(new WeaponSkill(getLocalString("fast.blade"), 10, WS_NORMAL, EARTH));
SWORD.addSkill(new WeaponSkill(getLocalString("burning.blade"), 30, WS_NORMAL, FIRE));
SWORD.addSkill(new WeaponSkill(getLocalString("red.lotus.blade"), 50, WS_EX, FIRE, WIND)); // EX
SWORD.addSkill(new WeaponSkill(getLocalString("flat.blade"), 75, WS_NORMAL, THUNDER));
SWORD.addSkill(new WeaponSkill(getLocalString("shining.blade"), 100, WS_NORMAL, EARTH));
SWORD.addSkill(new WeaponSkill(getLocalString("seraph.blade"), 125, WS_EX, EARTH)); // EX
SWORD.addSkill(new WeaponSkill(getLocalString("circle.blade"), 150, WS_NORMAL, WATER, THUNDER));
SWORD.addSkill(new WeaponSkill(getLocalString("spirits.within"), 175, WS_NORMAL));
SWORD.addSkill(new WeaponSkill(getLocalString("vorpal.blade"), 200, WS_EX, EARTH, THUNDER)); // EX
SWORD.addSkill(new WeaponSkill(getLocalString("swift.blade"), 225, WS_SP, UE_GRAVITATION)); // SP
SWORD.addSkill(new WeaponSkill(getLocalString("savage.blade"), 240, WS_NORMAL, UE_FRAGMENTATION, EARTH));

//debug("Database load complete.");
